@@ -1,14 +1,13 @@
 <?php
 /**
  *
- * @package MediaWiki
- * @subpackage SpecialPage
+ * @addtogroup SpecialPage
  */
 
 /**
- *
- * @package MediaWiki
- * @subpackage SpecialPage
+ * A special page looking for articles with no article linking to them,
+ * thus being lonely.
+ * @addtogroup SpecialPage
  */
 class LonelyPagesPage extends PageQueryPage {
 
@@ -16,7 +15,7 @@ class LonelyPagesPage extends PageQueryPage {
 		return "Lonelypages";
 	}
 	function getPageHeader() {
-		return '<p>' . wfMsg('lonelypagestext') . '</p>';
+		return wfMsgExt( 'lonelypagestext', array( 'parse' ) );
 	}
 
 	function sortDescending() {
@@ -29,7 +28,7 @@ class LonelyPagesPage extends PageQueryPage {
 	function isSyndicated() { return false; }
 
 	function getSQL() {
-		$dbr =& wfGetDB( DB_SLAVE );
+		$dbr = wfGetDB( DB_SLAVE );
 		list( $page, $pagelinks ) = $dbr->tableNamesN( 'page', 'pagelinks' );
 
 		return
@@ -58,4 +57,4 @@ function wfSpecialLonelypages() {
 	return $lpp->doQuery( $offset, $limit );
 }
 
-?>
+
