@@ -4,6 +4,6 @@
 -- 2004-01-25
 --
 
-ALTER TABLE recentchanges
+ALTER TABLE /*$wgDBprefix*/recentchanges
 	ADD INDEX new_name_timestamp(rc_new,rc_namespace,rc_timestamp);
 
