@@ -8,6 +8,7 @@ require_once( "$IP/maintenance/commandLine.inc" );
 
 foreach ( glob( "$IP/languages/messages/Messages*.php" ) as $filename ) {
 	$base = basename( $filename );
+	$m = array();
 	if ( !preg_match( '/Messages(.*)\.php$/', $base, $m ) ) {
 		continue;
 	}
@@ -42,4 +43,4 @@ foreach ( glob( "$IP/languages/messages/Messages*.php" ) as $filename ) {
 	print "\n\n";
 }
 
-?>
+
